/* global APP_STARTUP, APP_SHUTDOWN */
/* exported startup, shutdown, install, uninstall */

'use strict';

let bgProcess;

function startup(data, reason) {
	let {AddonManager} = Components.utils.import(
		'resource://gre/modules/AddonManager.jsm',
		null
	);

	let appShell = Components.classes['@mozilla.org/appshell/appShellService;1']
		.getService(Components.interfaces.nsIAppShellService);

	let onAddonData = function(addon) {
		let hDoc = appShell.hiddenDOMWindow.document;

		bgProcess = hDoc.documentElement.appendChild(
			hDoc.createElementNS('http://www.w3.org/1999/xhtml', 'iframe')
		);

		let host = addon.name.toLowerCase();
		let bgURI = 'chrome://' + host + '/content/background.html';

		// A way to send addon data synchronously to the background script
		bgProcess.src = bgURI + '#' + [addon.name, addon.version];
	};

	let onReady = function(e) {
		if (e) {
			this.removeEventListener(e.type, onReady);
		}

		AddonManager.getAddonByID(data.id, onAddonData);
	};

	if (reason !== APP_STARTUP) {
		onReady();
		return;
	}

	let ww = Components.classes['@mozilla.org/embedcomp/window-watcher;1']
		.getService(Components.interfaces.nsIWindowWatcher);

	ww.registerNotification({
		observe: function(win, topic) {
			if (topic !== 'domwindowopened') {
				return;
			}

			try {
				appShell.hiddenDOMWindow;
			} catch (ex) {
				return;
			}

			ww.unregisterNotification(this);
			win.addEventListener('DOMContentLoaded', onReady);
		}
	});
}

function shutdown(data, reason) {
	if (reason !== APP_SHUTDOWN) {
		bgProcess.parentNode.removeChild(bgProcess);
	}
}

function install() {
	// https://bugzil.la/719376
	Components.classes['@mozilla.org/intl/stringbundle;1']
		.getService(Components.interfaces.nsIStringBundleService)
		.flushBundles();
}

function uninstall() {}
