'use strict';

this.EXPORTED_SYMBOLS = ['docObserver'];

const {utils: Cu} = Components;
const {Services} = Cu.import('resource://gre/modules/Services.jsm', null);
const hostName = __URI__.match(/^[^:]+:\/\/(\w+)/)[1];

let docObserver = {
	uniqueSandboxId: 1,
	contentBaseURI: 'chrome://' + hostName + '/content/includes/',

	QueryInterface: (function() {
		let {XPCOMUtils} = Cu.import('resource://gre/modules/XPCOMUtils.jsm', {});

		return XPCOMUtils.generateQI([
			Components.interfaces.nsIObserver,
			Components.interfaces.nsISupportsWeakReference
		]);
	})(),

	register: function() {
		Services.obs.addObserver(this, 'content-document-global-created', true);
		// for the Options page only
		Services.obs.addObserver(this, 'chrome-document-global-created', true);
	},

	unregister: function() {
		Services.obs.removeObserver(this, 'content-document-global-created');
		Services.obs.removeObserver(this, 'chrome-document-global-created');
	},

	initContentScripts: function(win, js) {
		let Ci = Components.interfaces;
		let messager = win
			.QueryInterface(Ci.nsIInterfaceRequestor)
			.getInterface(Ci.nsIDocShell)
			.sameTypeRootTreeItem
			.QueryInterface(Ci.nsIDocShell)
			.QueryInterface(Ci.nsIInterfaceRequestor)
			.getInterface(Ci.nsIContentFrameMessageManager);
		let sandboxId = hostName + ':sb:' + this.uniqueSandboxId++;
		let sandbox;

		if (js) {
			try {
				let sandboxName = [
					win.location.href.slice(0, 100),
					win.document.title.slice(0, 100)
				].join(' | ');

				sandbox = Components.utils.Sandbox([win], {
					sandboxName: sandboxId + '[' + sandboxName + ']',
					sandboxPrototype: win,
					wantComponents: false,
					wantXHRConstructor: false
				});
			} catch (ex) {
				// This happens with media that are located under privileged addresses,
				// like on chrome: protocols, which we can freely pollute.
			}
		}

		if (!sandbox) {
			sandbox = win;
		}

		sandbox._sandboxId_ = sandboxId;
		sandbox._hostName_ = hostName;
		sandbox.sendAsyncMessage = messager.sendAsyncMessage;

		sandbox.addMessageListener = function(callback) {
			if (this._messageListener_) {
				this.removeMessageListener(
					this._sandboxId_,
					this._messageListener_
				);
			}

			this._messageListener_ = function(message) {
				callback(message.data);
			};

			messager.addMessageListener(this._sandboxId_, this._messageListener_);
		}.bind(sandbox);

		sandbox.removeMessageListener = function() {
			try {
				messager.removeMessageListener(this._sandboxId_, this._messageListener_);
			} catch (ex) {}

			this._messageListener_ = null;
		}.bind(sandbox);

		if (js) {
			let lss = Services.scriptloader.loadSubScript;
			lss(this.contentBaseURI + 'app.js', sandbox);
			lss(this.contentBaseURI + js, sandbox);
		}
	},

	onMouseEnter: function(e) {
		this.removeEventListener(e.type, docObserver.onMouseEnter, true);

		if (docObserver) {
			docObserver.initContentScripts(this.ownerDocument.defaultView, 'opener.js');
		}
	},

	onReady: function(e) {
		if (e) {
			this.removeEventListener('DOMContentLoaded', docObserver.onReady, true);
		}

		if (docObserver) {
			this.documentElement.addEventListener(
				'mouseenter',
				docObserver.onMouseEnter,
				true
			);
		}
	},

	observe: function(win) {
		if (!win) {
			return;
		}

		let doc = win.document;

		if (doc instanceof win.ImageDocument
			|| doc instanceof win.HTMLDocument
				&& doc.querySelector('body > video:only-child:not([src])')) {
			this.initContentScripts(win, 'viewer.js');
			return;
		}

		let loc = win.location;

		if (!/^(https?|f(tp|ile)):$/.test(loc.protocol)) {
			if (loc.protocol === 'chrome:' && loc.host === hostName) {
				this.initContentScripts(win);
			}

			return;
		}

		doc.addEventListener('DOMContentLoaded', docObserver.onReady, true);
	}
};

docObserver.register();
