'use strict';

var browser = {}, Port, _;

if (document instanceof window.HTMLDocument) {
	browser = document.documentElement || document.createElementNS('http://www.w3.org/1999/xhtml', 'div');

	if (!window.requestAnimationFrame) {
		window.requestAnimationFrame = function(cb) {
			return window.setTimeout(cb, 25);
		};
	}

	if (browser.style) {
		_ = browser.style;

		browser = {
			'wheel': 'onwheel' in browser ? 'wheel' : 'mousewheel',
			'transform': ('webkitTransform' in _ ? 'webkitT' : 't') + 'ransform',
			'transform_css': ('webkitTransform' in _ ? '-webkit-' : '') + 'transform',
			'transition_css': ('webkitTransition' in _ ? '-webkit-' : '') + 'transition',
			'transitionend': 'webkitTransition' in _ ? 'webkitTransitionEnd' : 'transitionend',
			'box-sizing_css': ('MozBoxSizing' in _ ? '-moz-' : ('WebkitBoxSizing' in _ ? '-webkit-' : '')) + 'box-sizing',
			'zoom-in': this.chrome || this.mx || this.safari ? '-webkit-zoom-in' : 'zoom-in'
		};
		_ = null;
	}
}

function buildNodes(host, nodes) {
	if (!host || !Array.isArray(nodes)) {
		return;
	}

	if (!nodes.length) {
		return host;
	}

	var doc = host.ownerDocument;
	var fragment = doc.createDocumentFragment();

	for (var i = 0, l = nodes.length; i < l; i++) {
		if (!nodes[i]) {
			continue;
		}

		if (typeof nodes[i] === 'string') {
			fragment.appendChild(doc.createTextNode(nodes[i]));
			continue;
		}

		var node = doc.createElement(nodes[i].tag);

		if (nodes[i].attrs) {
			for (var attr in nodes[i].attrs) {
				// bypass CSP
				if (attr === 'style') {
					node.style.cssText = nodes[i].attrs[attr];
				}
				else {
					node.setAttribute(attr, nodes[i].attrs[attr]);
				}
			}
		}

		if (nodes[i].nodes) {
			buildNodes(node, nodes[i].nodes);
		}
		else if (nodes[i].text) {
			node.textContent = nodes[i].text;
		}

		fragment.appendChild(node);
	}

	if (fragment.childNodes.length) {
		host.appendChild(fragment);
	}

	return host;
}

function mediaType(type) {
	if (type) {
		document.mediaType = type;
	}

	if (document.mediaType) {
		return document.mediaType;
	}

	var d = document, img = d.images && d.images.length === 1 && d.images[0];

	if (img && img.parentNode === d.body && img.src === window.location.href) {
		return document.mediaType = 'img';
	}

	d = d.querySelector(
		'body > video:first-child, body > div:only-child > video:first-child'
	);

	// Firefox doesn't handle redirects properly, so it gives the previous URL here
	if (d && (browser.firefox || d.currentSrc === window.location.href)) {
		return document.mediaType = 'video';
	}
}

browser.firefox = true;

Port = {
	listen: function(listener) {
		if (typeof listener === 'function') {
			this.listener = function(msg) {
				listener(JSON.parse(msg));
			};
			addMessageListener(this.listener);
		}
		else {
			removeMessageListener();
		}
	},

	send: function(msg) {
		sendAsyncMessage(_hostName_ + ':background', {
			listenerId: _sandboxId_,
			data: msg,
			origin: window.location.href
		});
	},

	toggleListener: function(e) {
		if (!Port.listener) {
			return;
		}

		if (e.type === 'pagehide') {
			removeMessageListener();
			return;
		}

		if (e.persisted) {
			addMessageListener(Port.listener);
		}
	}
};

window.addEventListener('pagehide', Port.toggleListener, true);
window.addEventListener('pageshow', Port.toggleListener, true);

if (location.protocol === 'chrome:' && location.hostname === _hostName_) {
	_ = (function() {
		var stringBundle = Components.classes['@mozilla.org/intl/stringbundle;1']
			.getService(Components.interfaces.nsIStringBundleService)
			.createBundle('chrome://' + location.host + '/locale/options.properties');

		return function(s) {
			try {
				return stringBundle.GetStringFromName(s);
			} catch (ex) {
				return s;
			}
		};
	})();

	browser.insertHTML = (function() {
		var Cc = Components.classes;
		var Ci = Components.interfaces;
		var io = Cc['@mozilla.org/network/io-service;1'].getService(Ci.nsIIOService);
		var parser = Cc['@mozilla.org/parserutils;1'].getService(Ci.nsIParserUtils);

		return function(node, html) {
			while (node.firstChild) {
				node.removeChild(node.firstChild);
			}

			node.appendChild(parser.parseFragment(
				html,
				parser.SanitizerAllowStyle,
				false,
				io.newURI(document.baseURI, null, null),
				document.documentElement
			));
		};
	})();
}

