'use strict';

var cfg, Tabs, Port, app;

if (location.hash) {
	app = location.hash.slice(1).split(',');
	app = {
		name: app[0],
		version: app[1]
	};
}


var Cc = Components.classes;
var Ci = Components.interfaces;
Components.utils['import']('resource://gre/modules/Services.jsm');

app.baseURI = 'chrome://' + location.host  + '/content/';

cfg = {
	PB: Services.prefs.getBranch('extensions.' + app.name + '.'),
	str: Cc['@mozilla.org/supports-string;1'].createInstance(Ci.nsISupportsString),
	getItem: function(k, fn) {
		try {
			fn(this.PB.getComplexValue(k, Ci.nsISupportsString).data);
		} catch (ex) {
			fn(null);
		}
	},
	setItem: function(k, v) {
		this.str.data = v;
		this.PB.setComplexValue(k, Ci.nsISupportsString, this.str);
	}
};

Tabs = {
	curWin: function() {
		return Services.wm.getMostRecentWindow('navigator:browser');
	},
	getSelected: function(cb) {
		var win = this.curWin();
		cb(win && win.gBrowser ? win.gBrowser.selectedTab : {});
	},
	create: function(p) {
		var win = this.curWin();

		if (win) {
			if (!/^[^:]{2,20}:/.test(p.url)) {
				p.url = app.baseURI + p.url;
			}

			win.gBrowser.loadOneTab(p.url, {inBackground: !p.active});
		}
	}
};

Port = {
	gmm: Cc['@mozilla.org/globalmessagemanager;1'].getService(Ci.nsIMessageListenerManager),
	frameScript: app.baseURI + 'frameScript.js',
	parseMessage: function(msg) {
		var listenerId = msg.data.listenerId;
		var messager = msg.target.messageManager;

		return {
			msg: msg.data.data,
			origin: msg.data.origin,
			postMessage: function(message) {
				messager.sendAsyncMessage(listenerId, JSON.stringify(message));
			}
		};
	},
	listen: function(fn) {
		if (typeof fn === 'function') {
			this.gmm.addMessageListener(location.host + ':background', fn);
			this.postMessage = fn;
		}
		else {
			this.gmm.removeMessageListener(
				location.host + ':background',
				this.postMessage
			);
			this.postMessage = null;
		}
	}
};

Port.gmm.loadFrameScript(Port.frameScript, true);

window.addEventListener('unload', function() {
	Port.gmm.removeMessageListener(location.host + ':background', Port.postMessage);
	Port.gmm.removeDelayedFrameScript(Port.frameScript);

	var URI = Port.frameScript.replace('Script', 'Module');
	var frameModule = {};
	Components.utils.import(URI, frameModule);
	frameModule.docObserver.unregister();
	Components.utils.unload(URI);

	// close extension tabs
	var winumerator = Services.wm.getEnumerator('navigator:browser');
	var gBrowser, tabs, i, extURI;

	while (winumerator.hasMoreElements()) {
		gBrowser = winumerator.getNext().gBrowser;
		tabs = gBrowser.tabs;
		i = tabs.length;

		while (i--) {
			extURI = tabs[i].linkedBrowser.currentURI;

			if (extURI.schemeIs('chrome') && extURI.host === location.host) {
				gBrowser.removeTab(tabs[i]);
			}
		}
	}
});
